@@ -19,7 +19,7 @@ from .keyword import Keyword, KeywordAssociation, Thesaurus
 from .person import Person, PersonRole, PersonAssociation
 from .license import License
 from .variable import Variable, Unit
-from .datasource import DataSource, DataSourceType
+from .datasource import DataSource, DataSourceType, DataType, SpatialScale, TemporalScale
 from .timeseries import TimeseriesPoint, TimeseriesPoint2D
 from .generic_data import DataPoint, DataPoint2D
 from .geometry_data import GeometryTimeseries, GenericGeometryData
